from abc import ABC, abstractmethod
from typing import Optional

from cpl_query.extension import List

from bot_data.model.realm import Realm
from bot_data.model.realm_moderator import RealmModerator


class RealmRepositoryABC(ABC):
    @abstractmethod
    def __init__(self):
        pass

    @abstractmethod
    def get_all(self) -> List[Realm]:
        pass

    @abstractmethod
    def get_realm_by_id(self, id: int) -> Realm:
        pass

    @abstractmethod
    def find_realm_by_id(self, id: int) -> Optional[Realm]:
        pass

    @abstractmethod
    def find_realm_by_name(self, name: str) -> Optional[Realm]:
        pass

    @abstractmethod
    def get_realms_by_server_id(self, id: int) -> List[Realm]:
        pass

    @abstractmethod
    def add_realm(self, realm: Realm):
        pass

    @abstractmethod
    def update_realm(self, realm: Realm):
        pass

    @abstractmethod
    def delete_realm(self, realm: Realm):
        pass

    @abstractmethod
    def get_realm_moderators(self) -> List[RealmModerator]:
        pass

    @abstractmethod
    def get_realm_moderator_by_id(self, id: int) -> RealmModerator:
        pass

    @abstractmethod
    def get_realm_moderators_by_realm_id(self, id: int) -> List[RealmModerator]:
        pass

    @abstractmethod
    def add_realm_moderator(self, realm: RealmModerator):
        pass

    @abstractmethod
    def update_realm_moderator(self, realm: RealmModerator):
        pass

    @abstractmethod
    def delete_realm_moderator(self, realm: RealmModerator):
        pass
