from typing import Optional

import discord
from cpl_discord.container import Role, CategoryChannel
from discord import Guild, Member

from bot_data.abc.realm_repository_abc import RealmRepositoryABC


class RealmUtils:
    def __init__(self, realms: RealmRepositoryABC):
        self._realms = realms

    @staticmethod
    def find_role_by_name(guild: Guild, name: str) -> Optional[Role]:
        for role in guild.roles:
            if role.name == name:
                return Role(role)
        return None

    @staticmethod
    def find_category_by_name(guild: Guild, name: str) -> Optional[CategoryChannel]:
        for category in guild.categories:
            if category.name == name:
                return CategoryChannel(category)
        return None

    @staticmethod
    def check_realm_role(guild: Guild, role: Role) -> bool:
        return role in guild.roles

    @staticmethod
    async def create_category(guild: Guild, name: str, role: Role, leader: Optional[Member]) -> CategoryChannel:
        overwrites = {
            guild.default_role: discord.PermissionOverwrite(view_channel=False, connect=False),
            role: discord.PermissionOverwrite(view_channel=True, connect=True),
        }
        if leader is not None:
            overwrites[leader] = discord.PermissionOverwrite(
                view_channel=True,
                connect=True,
                manage_channels=True,
                manage_permissions=True,
                create_instant_invite=True,
            )

            if leader.get_role(role.id) is None:
                await leader.add_roles(role)

        return await guild.create_category(name, overwrites=overwrites)

    @staticmethod
    async def assign_moderator(category: CategoryChannel, member: Member):
        await category.set_permissions(
            member,
            view_channel=True,
            connect=True,
            manage_channels=True,
            manage_permissions=True,
            create_instant_invite=True,
        )

    @staticmethod
    async def remove_moderator(category: CategoryChannel, member: Member):
        await category.set_permissions(
            member,
            view_channel=True,
            connect=True,
            manage_channels=False,
            manage_permissions=False,
            create_instant_invite=False,
        )

    def is_realm_moderator(self, member_id: int, realm_id: int) -> bool:
        return (
            self._realms.get_realm_moderators_by_realm_id(realm_id)
            .select(lambda x: x.user.discord_id)
            .contains(member_id)
        )
